@@ -3,16 +3,19 @@ use strict;
 use warnings;
 
 package XML::Atom::SimpleFeed;
-$XML::Atom::SimpleFeed::VERSION = '0.861';
+$XML::Atom::SimpleFeed::VERSION = '0.900';
 # ABSTRACT: No-fuss generation of Atom syndication feeds
 
 use Carp;
 use Encode ();
 use POSIX ();
 
+my @XML_ENC = 'us-ascii'; # use array because local($myvar) error but local($myvar[0]) OK
+                          # and use a lexical because not a public interface
+
 sub ATOM_NS           () { 'http://www.w3.org/2005/Atom' }
 sub XHTML_NS          () { 'http://www.w3.org/1999/xhtml' }
-sub PREAMBLE          () { qq(<?xml version="1.0" encoding="us-ascii"?>\n) }
+sub PREAMBLE          () { qq(<?xml version="1.0" encoding="$XML_ENC[0]"?>\n) }
 sub W3C_DATETIME      () { '%Y-%m-%dT%H:%M:%S' }
 sub DEFAULT_GENERATOR () { {
 	uri     => 'https://metacpan.org/pod/' . __PACKAGE__,
@@ -24,6 +27,8 @@ sub DEFAULT_GENERATOR () { {
 # superminimal XML writer
 # 
 
+sub xml_encoding { local $XML_ENC[0] = shift; &{(shift)} }
+
 my %XML_ESC = (
 	"\xA" => '&#10;',
 	"\xD" => '&#13;',
@@ -34,7 +39,7 @@ my %XML_ESC = (
 	'>'   => '&gt;',
 );
 
-sub xml_cref { Encode::encode 'us-ascii', $_[ 0 ], Encode::HTMLCREF }
+sub xml_cref { Encode::encode $XML_ENC[0], $_[0], Encode::HTMLCREF }
 
 sub xml_escape {
 	$_[0] =~ s{ ( [<>&'"] ) }{ $XML_ESC{ $1 } }gex;
@@ -104,16 +109,8 @@ sub simple_construct {
 
 sub date_construct {
 	my ( $name, $dt ) = @_;
-
-	if ( $dt !~ /[^0-9]/ ) {
-		require POSIX;
-		$dt = POSIX::strftime( W3C_DATETIME . 'Z', gmtime $dt );
-	}
-	elsif ( ref $dt and eval { $dt->can( 'strftime' ) } ) {
-		$dt = $dt->strftime( W3C_DATETIME . '%z' );
-		$dt =~ s!(\d\d)\z!$1:!;
-	}
-
+	eval { $dt = $dt->epoch }; # convert to epoch to avoid dealing with everyone's TZ crap
+	$dt = POSIX::strftime( W3C_DATETIME . 'Z', gmtime $dt ) unless $dt =~ /[^0-9]/;
 	xml_tag $name, xml_escape $dt;
 }
 
@@ -222,9 +219,10 @@ sub generator_element {
 		my $content = delete $arg->{ name };
 		xml_tag [ generator => map +( $_ => $arg->{ $_ } ), grep exists $arg->{ $_ }, qw( uri version ) ], xml_escape( $content );
 	}
-	else {
+	elsif( defined $arg ) {
 		xml_tag generator => xml_escape( $arg );
 	}
+	else { '' }
 }
 
 # tag makers are called with the name of the tag they're supposed to handle as the first parameter
@@ -303,19 +301,23 @@ sub container_content {
 #
 
 sub XML::Atom::SimpleFeed::new {
-	my $self = bless {}, shift;
-	my $arg = ( @_ and 'HASH' eq ref $_[0] ) ? shift : {};
+	my $self = bless { xml_encoding => $XML_ENC[0] }, shift;
 
-	$self->{ do_add_generator } = 1;
+	if ( my @i = grep { '-encoding' eq $_[$_] } grep { not $_ % 2 } 0 .. $#_ ) {
+		croak 'multiple encodings requested' if @i > 1;
+		( undef, my $encoding ) = splice @_, $i[0], 2;
+		$self->{ xml_encoding } = $encoding;
+	}
 
-	$self->feed( @_ ) if @_; # support old-style invocation
-
-	return $self;
+	@_ ? $self->feed( @_ ) : $self;
 }
 
 sub XML::Atom::SimpleFeed::feed {
 	my $self = shift;
 
+	my $have_generator;
+
+	local $XML_ENC[0] = $self->{ xml_encoding };
 	$self->{ meta } = container_content feed => (
 		elements    => \@_,
 		required    => [ qw( id title updated ) ],
@@ -324,11 +326,14 @@ sub XML::Atom::SimpleFeed::feed {
 		callback    => {
 			author    => sub { $self->{ have_default_author } = 1 },
 			updated   => sub { $self->{ global_updated } = $_[ 0 ] },
-			generator => sub { $self->{ do_add_generator } = 0 },
+			generator => sub { $have_generator = 1 },
 		},
 		default_upd => time,
 	);
 
+	$self->{ meta } .= $make_tag{ generator }->( generator => DEFAULT_GENERATOR )
+		unless $have_generator;
+
 	return $self;
 }
 
@@ -355,6 +360,7 @@ sub XML::Atom::SimpleFeed::add_entry  {
 	#       [MIMEREG], but is not an XML media type [RFC3023], does not
 	#       begin with "text/", and does not end with "/xml" or "+xml".
 
+	local $XML_ENC[0] = $self->{ xml_encoding };
 	push @{ $self->{ entries } }, xml_tag entry => container_content entry => (
 		elements    => \@_,
 		required    => \@required,
@@ -366,18 +372,9 @@ sub XML::Atom::SimpleFeed::add_entry  {
 	return $self;
 }
 
-sub XML::Atom::SimpleFeed::no_generator {
-	my $self = shift;
-	$self->{ do_add_generator } = 0;
-	return $self;
-}
-
 sub XML::Atom::SimpleFeed::as_string {
 	my $self = shift;
-	if( $self->{ do_add_generator } ) {
-		$self->{ meta } .= $make_tag{ generator }->( generator => DEFAULT_GENERATOR );
-		$self->{ do_add_generator } = 0;
-	}
+	local $XML_ENC[0] = $self->{ xml_encoding };
 	PREAMBLE . xml_tag [ feed => xmlns => ATOM_NS ], $self->{ meta }, @{ $self->{ entries } };
 }
 
@@ -390,7 +387,7 @@ sub XML::Atom::SimpleFeed::print {
 
 sub XML::Atom::SimpleFeed::save_file { croak q{no longer supported, use 'print' instead and pass in a filehandle} }
 
-! ! 'Funky and proud of it.';
+!!'Funky and proud of it.';
 
 __END__
 
@@ -404,7 +401,7 @@ XML::Atom::SimpleFeed - No-fuss generation of Atom syndication feeds
 
 =head1 VERSION
 
-version 0.861
+version 0.900
 
 =head1 SYNOPSIS
 
@@ -433,9 +430,9 @@ version 0.861
 
 =head1 DESCRIPTION
 
-This module provides a minimal API for generating Atom syndication feeds
-quickly and easily. It supports all aspects of the Atom format, but has no
-provisions for generating feeds with extension elements.
+This is a minimal API for generating Atom syndication feeds quickly and easily.
+It supports all aspects of the Atom format itself but has no mechanism for the
+inclusion of extension elements.
 
 You can supply strings for most things, and the module will provide useful
 defaults. When you want more control, you can provide data structures, as
@@ -445,13 +442,20 @@ documented, to specify more particulars.
 
 =head2 C<new>
 
-XML::Atom::SimpleFeed instances are created by the C<new> constructor, which
-takes a list of key-value pairs as parameters. The keys are used to create the
-corresponding L<"Atom elements"|/ATOM ELEMENTS>. The following elements are
-available:
+Takes a list of key-value pairs.
+
+Most keys are used to create corresponding L<"Atom elements"|/ATOM ELEMENTS>.
+To specify multiple instances of an element that may be given multiple times,
+pass multiple key-value pairs with the same key.
+
+Keys that start with a dash specify how the XML document will be generated.
+
+The following keys are supported:
 
 =over
 
+=item * C<-encoding> (I<omissible>, default C<us-ascii>)
+
 =item * L</C<id>> (I<omissible>)
 
 =item * L</C<link>> (I<omissible>, multiple)
@@ -478,14 +482,14 @@ available:
 
 =back
 
-To specify multiple instances of an element that may be given multiple times,
-simply list multiple key-value pairs with the same key.
-
 =head2 C<add_entry>
 
-This method adds an entry into the Atom feed. It takes a list of key-value
-pairs as parameters. The keys are used to create the corresponding
-L<"Atom Elements"|/ATOM ELEMENTS>. The following elements are available:
+Takes a list of key-value pairs,
+used to create corresponding L<"Atom elements"|/ATOM ELEMENTS>.
+To specify multiple instances of an element that may be given multiple times,
+pass multiple key-value pairs with the same key.
+
+The following keys are supported:
 
 =over
 
@@ -513,14 +517,6 @@ L<"Atom Elements"|/ATOM ELEMENTS>. The following elements are available:
 
 =back
 
-To specify multiple instances of an element that may be given multiple times,
-simply list multiple key-value pairs with the same key.
-
-=head2 C<no_generator>
-
-Suppresses the output of a default C<generator> element. It is not necessary to
-call this method if you supply a custom C<generator> element.
-
 =head2 C<as_string>
 
 Returns the XML representation of the feed as a string.
@@ -591,9 +587,10 @@ A L</Person Construct> denoting a contributor to the feed or entry.
 
 =head2 C<generator>
 
-The software used to generate the feed. Can be supplied as a string, or a hash
-with C<uri>, C<version> and C<name> keys. Defaults to reporting
-XML::Atom::SimpleFeed as the generator, which can be calling C<no_generator>.
+The software used to generate the feed. Can be supplied as a string
+or as a hash with C<uri>, C<version> and C<name> keys. Can also be undef to
+suppress the element entirely. If nothing is passed, defaults to reporting
+XML::Atom::SimpleFeed as the generator.
 
 =head2 C<icon>
 
@@ -703,9 +700,41 @@ using something like
  my $now = strftime '%Y-%m-%dT%H:%M:%SZ', gmtime;
 
 However, you can also simply pass a Unix timestamp (a positive integer) or an
-object that responds to a C<strftime> method call (such as
-a L<Time::Piece|Time::Piece> or L<DateTime|DateTime> instance). Make sure that
-the timezone reported by such objects is correct!
+object that responds to an C<epoch> method call. (Make sure that the timezone
+reported by such objects is correct!)
+
+The following datetime classes from CPAN are compatible with this interface:
+
+=over 4
+
+=item * L<Time::Piece|Time::Piece>
+
+=item * L<DateTime|DateTime>
+
+=item * L<Time::Moment|Time::Moment>
+
+=item * L<Panda::Date|Panda::Date>
+
+=item * L<Class::Date|Class::Date>
+
+=item * L<Time::Object|Time::Object> (an obsolete precursor to L<Time::Piece|Time::Piece>)
+
+=back
+
+The following are not:
+
+=over 4
+
+=item * L<DateTime::Tiny|DateTime::Tiny>
+
+This class lacks both an C<epoch> method or any way to emulate one E<ndash> as
+well as any timezone support in the first place.
+
+=item * L<Date::Handler|Date::Handler>
+
+This class has a suitable methodE<hellip> but sadly, calls it C<Epoch>.
+
+=back
 
 =head2 Person Construct
 
